==> advent_of_code_wim-2021.10-py3-none-any.whl: aoc_wim/aoc2021/q10.py <==
"""
--- Day 10: Syntax Scoring ---
https://adventofcode.com/2021/day/10
"""
from aocd import data

opens = "([{<"
closes = ")]}>"
weight_a = dict(zip(closes, [3, 57, 1197, 25137]))
weight_b = dict(zip(closes, range(1, 5)))
flip = dict(zip(opens + closes, closes + opens))


class ParserError(Exception):
    def __init__(self, line, expected, found):
        self.line = line
        self.expected = expected
        self.found = found
        self.syntax_error_score = weight_a[found]

    def __str__(self):
        return f"{self.line} - Expected {self.expected}, but found {self.found} instead."


class Parser:
    def __init__(self):
        self.pos = 0
        self.stack = []

    def __call__(self, line):
        while self.pos < len(line):
            found = line[self.pos]
            if found in opens:
                self.stack.append(found)
            elif found in closes:
                expected = flip[self.stack[-1]]
                if found != expected:
                    raise ParserError(line, expected, found)
                self.stack.pop()
            self.pos += 1

    @property
    def autocompletion(self):
        return "".join(flip[char] for char in reversed(self.stack))

    @property
    def autocomplete_score(self):
        result = 0
        for char in self.autocompletion:
            result *= 5
            result += weight_b[char]
        return result


a = 0
bs = []
for line in data.splitlines():
    parser = Parser()
    try:
        parser(line)
    except ParserError as err:
        a += err.syntax_error_score
    else:
        bs.append(parser.autocomplete_score)

bs.sort()
b = None
if bs:
    b = bs[len(bs)//2]

print("part a:", a)
print("part b:", b)

==> advent_of_code_wim-2021.17-py3-none-any.whl: aoc_wim/aoc2021/q10.py <==
"""
--- Day 10: Syntax Scoring ---
https://adventofcode.com/2021/day/10
"""
from aocd import data
from statistics import median


class ParserError(Exception):
    def __init__(self, line, expected, found):
        self.line = line
        self.expected = expected
        self.found = found
        self.syntax_error_score = [3, 57, 1197, 25137][")]}>".index(found)]

    def __str__(self):
        return f"{self.line} - Expected {self.expected}, but found {self.found} instead."


class Parser:

    lookup = dict(zip("([{<", ")]}>"))

    def __init__(self):
        self.pos = 0
        self.stack = []

    def __call__(self, line):
        while self.pos < len(line):
            found = line[self.pos]
            if found in self.lookup:
                self.stack.append(found)
            else:
                expected = self.lookup[self.stack[-1]]
                if found != expected:
                    raise ParserError(line, expected, found)
                self.stack.pop()
            self.pos += 1

    @property
    def autocompletion(self):
        return "".join(self.lookup[char] for char in reversed(self.stack))

    @property
    def autocomplete_score(self):
        result = 0
        for char in self.autocompletion:
            result *= 5
            result += ")]}>".index(char) + 1
        return result


a = b = 0
bs = []
for line in data.splitlines():
    parser = Parser()
    try:
        parser(line)
    except ParserError as err:
        a += err.syntax_error_score
    else:
        bs.append(parser.autocomplete_score)

if bs:
    b = median(bs)

print("part a:", a)
print("part b:", b)
